# Configure

```
cp bin/smartdata/Config.php.template bin/smartdata/Config.php
```

- If you are confident enough, you can:
  - Edit `bin/smartdata/Config.php` to customize your environment.
  - Edit `docker/variable.env` to customize it, matching with the file above.

# Running

```
docker compose up
```

# Writing Tests

- Create a file `./tests/xx-name.tests.py` (has to end with ".tests.py")
- Check the other tests and copy the initialization part, in order to create domain, create credentials, create series and put some data.

```python
# Example
# from common.PlatformManagement import PlatformManagement
from common.SmartdataClient import SmartdataClient
import time
# Initializing Test Environment
service = PlatformManagement()
domain = 'smartdatatest' + str(int(time.time()))
assert service.createDomain(domain), 'Domain was not created'
cert = service.generateCredentials(domain)
assert cert['serial'], 'Cert was not created'
with open("/tmp/client.key", "w", encoding="utf-8") as file:
    file.write(str(cert['key']))
with open("/tmp/client.crt", "w", encoding="utf-8") as file:
    file.write(str(cert['pem']))
# Testing
unit = 0x84925924
value = 5
client = SmartdataClient()

serie = client.createSeries(unit, 0, 0, 0, 100, 0, 10, 0, 'sig')
assert serie, "SERIE should return True"
put = client.put(unit, value, 1, 1, 1, 3, 0, 'sig')
assert put, f"PUT should return something, instead of {put}"
get = client.get(unit, 1, 1, 1, 10, 0, 10, 0, 'sig')
assert get != [], f"GET should return something, instead of {get}"
```
