<?php

namespace SmartData;

require_once('../bin/smartdata/SmartAPI.php');
require_once('../bin/smartdata/Logger.php');
require_once('../bin/smartdata/SmartData.php');
require_once('../bin/smartdata/MultiSmartData.php');


use SmartData\Exception\{AuthenticationException, BadRequestException, InternalException};

http_response_code(HttpStatusCode::BAD_REQUEST);

try {
    $_REQUEST['stats'] = array("arrived" => intval(1000000 * microtime(true)));
    $content = file_get_contents('php://input');
    $values = json_decode($content, false, 512, JSON_BIGINT_AS_STRING);
    $_REQUEST['stats']['firts_decoding'] = intval(1000000 * microtime(true));
    $error = json_last_error();
    if ($error === JSON_ERROR_NONE) {
        if (property_exists($values, "credentials")) {
            $credentials = Credentials::fromJson($values->credentials);
            $backend = new Backend($credentials);
        } else {
            $backend = new Backend();
        }
        $data = MultiSmartData::multi_values($values);
        $_REQUEST['stats']['mv_decoding'] = intval(1000000 * microtime(true));
        if (is_array($data)) {
            if ($data[0]->version == SmartData::STATIC_VERSION) {
                if (!$backend->insert_multiple(...$data)) {
                    throw new Exception\InsertionFailedException("JSON MultiValue SmartData Insertion Failed");
                }
            } else {
                if (!$backend->track_multiple(...$data)) {
                    throw new Exception\InsertionFailedException("JSON MultiValue SmartData Insertion Failed");
                }
            }
        } else if ($data instanceof MultiSmartData) {
            if ($data->version == SmartData::STATIC_VERSION) {
                if (!$backend->insert_blob($data)) {
                    throw new Exception\InsertionFailedException("JSON MultiValue SmartData 1.1 Blob Insertion Failed");
                }
            } else {
                if (!$backend->insert_tracker_blob($data)) {
                    throw new Exception\InsertionFailedException("JSON MultiValue SmartData 1.2 Blob Insertion Failed");
                }
            }
        }
        $_REQUEST['stats']['before_returning'] = intval(1000000 * microtime(true));
        $labels = array_keys($_REQUEST['stats']);
        Logger::debug("STATS: " . implode(" ; ", $labels));
        Logger::debug("STATS: " . implode(" ; ", $_REQUEST['stats']));
    } else {
        // Is Binary
        $data = MultiSmartData::unpackMultiValue($content);
        $db = new Backend();
        if ($data instanceof MultiSmartData) {
            if (!$db->insert_blob($data)) {
                throw new Exception\InsertionFailedException("JSON MultiValue SmartData Blob Insertion Failed");
            }
        } else {
            if (!$db->insert_multiple(...$data)) {
                throw new Exception\InsertionFailedException("Binary MultiValue SmartData Insertion Failed");
            }
        }
    }
} catch (Exception\CustomException $e) {
    http_response_code($e->getHTTPCodeError());
    error_log($e);
    header('X-Message: ' . $e->getMessage(), false);
    return false;
} catch (\Exception $e) {
    http_response_code(HttpStatusCode::BAD_REQUEST);
    error_log($e);
    header('X-Message: ' . $e->getMessage(), false);
    return false;
}

http_response_code(HttpStatusCode::NO_CONTENT);
