<?php

namespace SmartData;

require_once(__DIR__ . '/Exception.php');
require_once(__DIR__ . '/Packer.php');
require_once(__DIR__ . '/Unit.php');
require_once(__DIR__ . '/Logger.php');

class Description
{
    public $version;
    private $unit;
    public $x;
    public $y;
    public $z;
    public $dev;
    public $signature;
    public $type;
    public $period;
    public $description;

    public function __construct(
        $v,
        $u,
        $x,
        $y,
        $z,
        $dev,
        $description,
        $type = 'OLD',
        $period = 0,
        $signature = 0
    ) {
        $tmp_v = $v;
        if (!(is_numeric($tmp_v) && is_int($tmp_v + 0))) {
            $tmp_v = explode(".", $tmp_v);
            $tmp_v = ($tmp_v[0] << 4) + $tmp_v[1];
        }

        $this->version      = $tmp_v;
        $this->unit         = ($u instanceof Unit) ? $u : Unit::interpret($u);
        $this->x            = $x;
        $this->y            = $y;
        $this->z            = $z;
        $this->dev          = $dev;
        $this->signature    = $signature;
        $this->type         = $type;
        $this->period       = $period;
        $this->description  = $description;
    }

    public function unit()
    {
        return $this->unit;
    }

    public function __get($property)
    {
        if (property_exists($this, $property)) {
            switch ($property) {
                case 'unit':
                    if ($this->unit instanceof Unit)
                        return $this->unit->cod;
                    else
                        return $this->unit;
                default:
                    return $this->$property;
            }
        }
    }

    public function toArray()
    {
        $json = array();
        $json['version']     = $this->version;
        $json['unit']        = $this->unit->cod;
        $json['x']           = $this->x;
        $json['y']           = $this->y;
        $json['z']           = $this->z;
        $json['dev']         = $this->dev;
        $json['signature']   = $this->t0;
        $json['type']        = $this->type;
        $json['period']      = $this->period;
        $json['description'] = $this->description;
        return $json;
    }

    public function toJson()
    {
        $json = json_encode($this->toArray());
        $json = preg_replace('/:"([0-9]*)"/', ':$1', $json);
        return $json;
    }

    public static function fromJson(\stdClass $json)
    {

        $tp   = $json->type ?? 'OLD';
        $per  = $json->period ?? 0;
        $sd   = new self(
            $json->version,
            $json->unit ?? 0,
            $json->x,
            $json->y,
            $json->z,
            $json->dev ?? 0,
            $json->description,
            $tp,
            $per,
            $json->signature ?? 0
        );
        return $sd;
    }

    public function __toString()
    {
        $string  = "{";
        $string .= "v={$this->version}, ";
        $string .= "u={$this->unit}, ";
        $string .= "x={$this->x}, ";
        $string .= "y={$this->y}, ";
        $string .= "z={$this->z}, ";
        $string .= "dev={$this->dev}, ";
        $string .= "signature={$this->signature}, ";
        $string .= "type={$this->type}, ";
        $string .= "period={$this->period}, ";
        $string .= "description={$this->description}, ";
        if ($this->version == SmartData::MOBILE_VERSION) // Remove.. Now we have the classs "Tracker" for mobile version
            $string .= ",signature={$this->signature}";
        $string .= '}';
        return $string;
    }
}
