#!/usr/bin/php
<?php
namespace SmartData;
require_once( __DIR__ . '/../Config.php');
require_once( __DIR__ . '/../Backend.php');

if ($argc < 2) {
   error_log('Sintaxe: '.$argv[0].' path_to_files');
   exit();
}

$names = ["TUR_DIST_ABERT", "GER_SYN_MWA", "TUR_MGT_49_1", "TUR_MGT_49_2", "GER_SYN_KVAB", "GER_SYN_MVARA", "LUB_CIRC_49", "RT_MB_2_IEX", "RT_MED_VEX", "RV_MED_15", "TUR_ROTOR_ABERT"];

$units = [4160749569, 3835336996, 3834792292, 3834792292, 3835336484, 3835336996, 3834792292, 3835336996, 3835336484, 3835050276, 4160749569];

$conversion = ["PCT","MW","C","C","V","MW","C","MW","V","MS","PCT"];

$max = count($names);

function convert ($value, $unit)
{
    if ($unit == "MCA") {
        return $value * 9806.05;  // MCA to Pa
    }
    if ($unit == "BAR") {
        return $value * 100000; // Bar to Pa
    }
    if ($unit == "C") { // C to K
        return $value + 273.15;
    }
    if ($unit == "LMIN") {
        return $value * 60000; // l/min to M3/s
    }
    if ($unit == "MM") {
        return $value / 1000; // MM to M
    }
    if ($unit == "MW") {
        return $value * 1000000; // MWatt to Watt
    }
    return $value;
}

$BATCH_SIZE = 1000;
$path_name =  $argv[1];

$domain   = 'aqtech_hydro';
$username = 'aqtech_hydro';
$passwd   = 'P82$kl#kmz&74';

$fileList = glob($path_name."/*");

foreach ($fileList as $file_name) {
    $file = basename($file_name,".csv");
    echo("Processing file: " . $file);
    $row = 0;
    $i = 0;

    $conversor = "XXX";

    while ($i < $max && $names[$i] != $file) $i++;
    if ($i >= $max || $units[$i] == 0) { // $conversion[$i] == "XXX") {
        echo("Unable to import file ".$file." : no conversion found. " . $i . " " . $max . " " . $conversion[$i] . "\n");
    } else if ($units[$i] == 0) {
        echo("Unable to import file ".$file." : SI unit not defined.\n");
    } else {
        $conversor = $conversion[$i];
        $device = 1000+$i;
        if (($handle = fopen($file_name, "r")) == FALSE) {
            error_log('Failed to open file: '.$file_name."  Skiping...\n");
            continue;
        }

        $cred = new Credentials($domain, $username, $passwd);
        $backend = new Backend_V1_1($cred, true);

        $batch = array();
        $series = array();
        $data  = fgetcsv($handle, 500, ";"); //  discard header (first) line
        $data  = fgetcsv($handle, 500, ";"); //  get first data line

        $ts = ((int) $data[5]) / 1000000;
        $month = date("m", $ts);
        $year = (int) date("Y", $ts);
        $first_day = strtotime($year."-".$month."-01 00:00:00");
        $ini_date = $first_day * 1000000;
        $last_day = date("Y-m-t",$first_day);
        $end_date = strtotime($last_day." 23:59:59")*1000000 + 999999;
        $unt = $units[$i];
        $srs = new Series((float) $data[0], $unt, (int) $data[2], (int) $data[3], (int) $data[4], 0, $ini_date, $end_date, (int) $data[6]);
        error_log("Creating series to unit: ".$unt);
        $backend->create( $srs );
        array_push($series, $srs );
        $value = convert( (float) $data[8], $conversor);
        $smartdata = new StaticSmartData($unt, $value, (int) $data[7], 0, (int) $data[2] ,(int) $data[3], (int) $data[4], (int) $data[5], (int) $data[6]);
        array_push($batch, $smartdata);

        $batch_count = 1;
        $inserts_count = 0;
        $error_count = 0;

        while (($data = fgetcsv($handle, 500, ";")) !== FALSE) {
            $row++;
            $value = convert( (float) $data[8], $conversor);

            $ts = ((int) $data[5]) / 1000000;
            $next_month = date("m", $ts);
            $next_year  = (int) date("Y", $ts);
            if ($next_month != $month || $next_year != $year) {
               $ok = TRUE;
               if (count($batch) > 0) {
                   echo('Inserting  batch 1 ...');
                   if (! $backend->insertBatch($series[0], ...$batch))  {
                       $ok = FALSE;
                       $error_count += 1;
                   }
               }
               $month = $next_month;
               $year = $next_year;
               $first_day = strtotime($year."-".$month."-01 00:00:00");
               $ini_date = $first_day * 1000000;
               $last_day = date("Y-m-t",$first_day);
               $end_date = strtotime($last_day." 23:59:59")*1000000 + 999999;
               $srs = new Series(17, $unt, (int) $data[2], (int) $data[3], (int) $data[4], 0, $ini_date, $end_date, $device);
               //error_log( $srs->toJson() );
               $backend->create( $srs );
               $series = array();
               unset($batch);
               $batch = array();
               array_push($series, $srs );
               $batch_count = 0;
            }
            $smartdata = new StaticSmartData($unt, $value, (int) $data[7], 0, (int) $data[2] ,(int) $data[3], (int) $data[4], (int) $data[5], $device);
            //error_log($smartdata->toJson());
            array_push($batch, $smartdata);
            $batch_count++;
            if ($batch_count >= $BATCH_SIZE) {
                // send  batch  to cassandra
                $ok = TRUE;
                echo('Inserting  batch 2 ...');
                if (! $backend->insertBatch($series[0], ...$batch))  {
                    $ok = FALSE;
                    $error_count += 1;
                }
                if ( !$ok ) {
                    echo 'Insertion failed for batch insert number: ' . strval($inserts_count) . "\n";
                }
                $inserts_count++;
                unset($batch);
                $batch = array();
                $batch_count = 0;
            }
        } // while
        if ($batch_count > 0) {
            $ok = TRUE;
            echo('Inserting  batch 3 ...');
            if (! $backend->insertBatch($series[0], ...$batch))  {
                $ok = FALSE;
                $error_count += 1;
            }
            if ( !$ok ) {
                echo 'Insertion failed for batch insert number: ' . strval($inserts_count) . "\n";
            }
        }
        echo("Finished file " . $file_name . " with errors: " . $error_count) . "\n";
    }
}
?>
