#!/usr/bin/php
<?php
namespace SmartData;
require_once( __DIR__ . '/../Config.php');
require_once( __DIR__ . '/../Backend.php');

if ($argc < 6) {
   error_log('Sintaxe: '.$argv[0].' x y z mass path_to_files');
   exit();
}

$BATCH_SIZE = 1000;
$x = (int) $argv[1];
$y = (int) $argv[2];
$z = (int) $argv[3];
$g = (int) $argv[4];

$path_name =  $argv[5];

$accel   = 2224433444;
$voltage = 2224723748;
$hertz   = 2147485184;
$mass    = 2224212260;
$units = array($accel, $accel, $accel, $accel, $accel, $accel, $hertz, $voltage, $hertz, $mass);

$domain = 'mafaulda';
$username = 'mafaulda';
$passwd = 'quino';

$fileList = glob($path_name."/*");
$dttm = 1600128000000000;

foreach ($fileList as $file_name) {
    $rotation = (float) basename($file_name,".csv");
    $row = 0;

    if (($handle = fopen($file_name, "r")) == FALSE) {
        error_log('Failed to open file: '.$file_name.'  Skiping...\n');
        continue;
    }
    $cred = new Credentials($domain, $username, $passwd);
    $backend = new Backend_V1_1($cred, true);
    $batch = array();
    $series = array();
    for  ($s = 0; $s < 10; $s++) {
        array_push($series, new Series(SmartData::STATIC_VERSION, $units[$s], $x, $y, $z, 0, 1600128000000000, 1600256000000000, $s));
        array_push($batch, array());
    }
    $batch_count = 0;
    $inserts_count = 0;
    $error_count = 0;
    while (($data = fgetcsv($handle, ",")) !== FALSE) {
        $num = count($data);
        $row++;
        $dev = 0;
        for ($dev = 0; $dev < 8; $dev++) {
	   $smartdata = new StaticSmartData($units[$dev], ((float) $data[$dev]), 0, 0, $x, $y, $z , $dttm, $dev);
           array_push($batch[$dev], $smartdata);
        } // for
        $dev = 8; //  the rotation (hertz) of the experiment
        $smartdata = new StaticSmartData($units[8], $rotation, 0, 0, $x, $y, $z , $dttm, $dev);
        array_push($batch[8], $smartdata);
        $dev = 9; //  the mass used to induce unbalanced rotation.
        $smartdata = new StaticSmartData($units[9], $g, 0, 0, $x, $y, $z , $dttm, $dev);
        array_push($batch[9], $smartdata);
        $dttm = $dttm + 20;
        $batch_count++;
        if ($batch_count >= $BATCH_SIZE) {
            // send  batch  to cassandra
            $new_batch = array();
            $ok = TRUE;
            for ($i = 0; $i < 10; $i++) {
                if (! $backend->insertBatch($series[$i], ...$batch[$i]))  {
                    $ok = FALSE;
                    $error_count += 1;
                }
                array_push($new_batch, array());
            }
            if ( !$ok ) {
                echo 'Insertion failed for batch insert number: ' . strval($inserts_count) . "\n";
            }
            $inserts_count++;
            unset($batch);
            $batch = $new_batch;
            $batch_count = 0;
        }
    } // while
    echo("Finished file " . $file_name . " with errors: " . $error_count) . "\n";
}
?>
