
'''
Importing "Consolidated" data, that was wrongly imported from BLOBs, but must be searched in TrendSignals
'''
import psycopg2
from datetime import timedelta, datetime
import struct, sys, os
from subprocess import Popen, PIPE, STDOUT

DEPLOYED = False # True  # disable this flag to enable prints and disable insert at the DB
file_num = 1

devs = {67:14 , 68:15, 69:16, 70:17}

if (len(sys.argv) == 4):
    x = int(sys.argv[1])
    y = int(sys.argv[2])
    z = int(sys.argv[3])
else:
    raise Exception("Missing parameters, expected 4 (5-filename), but only %d were found."%(len(sys.argv)-1))

def query_postgress( _id):
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech_05_2021', user='aqtech', password='KdeiBmtewpt')

    cur = conn.cursor()

    print("query running ", _id)

    # How to speed up by removing JOINs: (Slow speed up, the number of lines is usually small, blob size is the slowing factor)
    # (1) SystemSampleInterval seems to be a fixed value (at least it is for this dataset). The sampling interval can be defined by 5e-6 * "SignalSettings"."SampleFactor". (Remove last JOIN)
    # (2) SignalSettings is a small table, a query could be executed only once to get its content. Its information can be accessed locally after querying each one of the Blobs (time in RAM vs. SSH connection). (Remove First JOIN)

    cur.execute('SELECT "TrendSignalSettingId", "DateTime", "Value", "Unit" FROM (SELECT * FROM "AQT"."TrendSignalRecords" WHERE "TrendSignalSettingId" IN (SELECT "Id" FROM "AQT"."TrendSignalSettings" WHERE "TrendSettingId" = 1)) AS "A" LEFT JOIN "AQT"."TrendSignalSettings" AS "B" ON "A"."TrendSignalSettingId" = "B"."Id" where "B"."Id" = %d order by "DateTime" '%(_id))

    return cur.fetchall()

def convert_to_si(unit, val):
    switcher = {        # SI UNIT  , Conversion         | SI UNITS
        "rpm":  lambda: [0xE4923924, val/60.0],         # SI - freq - Hz --> 0xE4923924 1/sec
        "kVA":  lambda: [0xE49A9924, val*1000],         # 1000 * V * A = m^2*kg*s^⁻3*A^-1 * A = W
        "V":    lambda: [0xE49A9724, val],              # m^2*kg*s^⁻3*A^-1
        "m":    lambda: [0xE4964924, val],              # m
        "um":   lambda: [0xE4964924, val/1000000],      # m
        "%":    lambda: [0xF8000001, val],              # SI
        "bar":  lambda: [0xE48EA924, val*100000],       # Pa
        "MVAr": lambda: [0xE49A9924, val*1000000],      # 1000000 * V * A * r = W
        "g":    lambda: [0xE4962924, val*9.80665],      # g =  9.8 m/s^2
        "MW":   lambda: [0xE49A9924, val*1000000],      # m^2*kg*s^⁻3
        "A":    lambda: [0xE4924B24, val],              # A
        "oC":   lambda: [0xE4924964, val+273.15],       # K
        "mm":   lambda: [0xE4964924, val/1000],         # m
        "mm/s": lambda: [0xE4963924, val/1000],         # m/s
        "" :    lambda: [0x00000001, val]               # Digital switch
    }

    func = switcher.get(unit, lambda: ["Error", 0])
    return func()

def read_blob_float(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/4)
    for k in range(nv):
        lv.append(
            struct.unpack('f',bytearray(blob[k*4:(k*4+4)]))[0]
            )
    return lv

def read_blob_double(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/8)
    for k in range(nv):
        lv.append(
            struct.unpack('d',bytearray(blob[k*8:(k*8+8)]))[0]
            )
    return lv

def read_blob_bool(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob)))
    for k in range(nv):
        lv.append(
            struct.unpack('?',bytearray(blob[k:(k+1)]))[0]
            )
    return lv

def insert_data(rows, _dev, series_exists = True):
    # rows[i] =  [0] "Id", [1] "DateTime", [2] "Raw", [3] "SignalType", [4] "Unit", [5] "SamplingInterval", [6] "dev"
    # Steps:
    # 1. Convert data from blobl to bool/float/double list
    # 2. Convert data values in list to SI unit equivalent
    # 3. Write touple (value,ts) as a csv
    # 4. Write final string to a file and call PHP, when PHP returns, delete file
    # 4.1. PHP file name is "single-series-csv-import.php"
    # 4.2. PHP function is "insert_csv(filename, unit, x, y, z, dev, t0, tf, series_exist)"
    global file_num
    if (rows[0][3] == 'xx'):
        return 'Finished'

    unit = convert_to_si(rows[0][3],0)[0]   # same unit for every data (each select is done for the same SignalSettingId)
    if (unit == "Error"):
         return "Not expected unit! rows[?][unit] = %s"%(rows[0][3])
    else:
        unit = int(unit)

    #dev  = int(rows[0][6])                       # same dev for every data (each select is done for the same SignalSettingId == dev)
    dev = _dev
    str_csv = ""
    t0 = None
    tf = None
    if (DEPLOYED):
        f = open("to_insert.csv","w")
    else:
        f = open("to_insert"+str(file_num)+".csv", "w")
        file_num += 1
    for i, r in enumerate(rows):
        dthr = r[1]
        if t0 is None:
            t0 = int(dthr.timestamp()*1000000)
        else:
            tf = int(dthr.timestamp()*1000000)
        value = float(r[2])
        unit_siv = convert_to_si(r[3], value)
        if (str(unit_siv[0]) == "Error"):
            return "Not expected unit! rows[%d][unit] = %s"%(i, r[4])
        str_csv = str(int(dthr.timestamp()*1000000))+','+str(unit_siv[1])+','+str(dev)+'\n'
        if (i == len(rows)-1) :
            str_csv = str_csv[:-1] #remove last "\n"
        f.write(str_csv)
    f.flush()
    f.close()
    if (DEPLOYED):
        if (series_exists):
            os.system("./single-series-csv-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(1)+" >> importer_php.log")
        else:
            os.system("./single-series-csv-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(0)+" >> importer_php.log")
        os.remove("to_insert.csv")
        return "Finished"
    else:
        print("./single-series-csv-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(0)+" >> importer_php.log")
        print(str_csv[:-1])
        return "OK"

#settings_size = 122+1
#settings_size = 60
#settings_size = 56

series_exists = False
#for setting in range(111, settings_size):
ks = devs.keys()
for _dev in ks:
    _id = devs[ _dev ]
    rows = query_postgress(_id)
    print("len rows:",len(rows))
    if (len(rows) > 0):
        out = insert_data(rows, _dev, series_exists)
        if ("Not expected unit!" in out):
            print(out)
            break
        print(out)
