import psycopg2
from datetime import timedelta, datetime
import struct, sys, os
from subprocess import Popen, PIPE, STDOUT

DEPLOYED = True  # disable this flag to enable prints and disable insert at the DB
file_num = 1

if (len(sys.argv) == 4):
    x = int(sys.argv[1])
    y = int(sys.argv[2])
    z = int(sys.argv[3])
else:
    raise Exception("Missing parameters, expeted 3 (4-filename), but only %d were found."%(len(sys.argv)-1))

'''
"AQT"
-------------------------------
"SignalRecordsBlobs"
Id | SignalRegisterId | SignalSettingId | DateTime | Raw
-------------------------------
"SignalSettings"
Id | MonitoringElementId | SignalRecorderId | Name | Color | SampleFactor | SignalType | Unit
-------------------------------
"SignalRegisters"
Id | SignalRecorderId | StartDateTime | EndDateTime | IsSequentialRegister | SequentialNumberOfRegister | SystemSampleInterval
-------------------------------
"SignalRecorder"
Id |      Name      | SettingId 
----+----------------+-----------
  1 | Vibracao       |         1
  2 | Partida_Parada |         1
  1 | Vibracao       |         1
  2 | Partida_Parada |         1
(4 rows)
-------------------------------
test query (light version -- limiting time range and settingid)
select distinct("Blobs"."Id"), "Blobs"."DateTime", "Blobs"."Raw", "Settings"."SignalType", "Settings"."Unit", "Registers"."SystemSampleInterval"*"Settings"."SampleFactor" as "SamplingInterval" from "AQT"."SignalRecordsBlobs" as "Blobs" inner join "AQT"."SignalSettings" as "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" join "AQT"."SignalRegisters" as "Registers" ON "Blobs"."SignalRegisterId" = "Registers"."Id" where "Blobs"."SignalSettingId" = 2 and "Blobs"."DateTime" between '2019-10-28 00:00:00' and '2019-12-28 00:00:00';
'''


def query_postgress_for_rows_ids(SettingsId, t0, t1):
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech', user='postgres', password='KdeiBmtewpt')

    cur = conn.cursor()

    print("querying ids: ", SettingsId, t0, t1)

    # How to speed up by removing JOINs: (Slow speed up, the number of lines is usually small, blob size is the slowing factor)
    # (1) SystemSampleInterval seems to be a fixed value (at least it is for this dataset). The sampling interval can be defined by 5e-6 * "SignalSettings"."SampleFactor". (Remove last JOIN)
    # (2) SignalSettings is a small table, a query could be executed only once to get its content. Its information can be accessed locally after querying each one of the Blobs (time in RAM vs. SSH connection). (Remove First JOIN)

    cur.execute('SELECT DISTINCT("Blobs"."Id") FROM "AQT"."SignalRecordsBlobs" AS "Blobs" JOIN "AQT"."SignalSettings" AS "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" WHERE "Blobs"."SignalSettingId" = %d AND "Blobs"."DateTime" BETWEEN \'%s\' AND \'%s\' '%(SettingsId, t0, t1))

    return cur.fetchall()

def query_postgress(SettingsId, t0, t1, _id):
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech', user='postgres', password='KdeiBmtewpt')

    cur = conn.cursor()

    print("query running", SettingsId, t0, t1, _id)

    # How to speed up by removing JOINs: (Slow speed up, the number of lines is usually small, blob size is the slowing factor)
    # (1) SystemSampleInterval seems to be a fixed value (at least it is for this dataset). The sampling interval can be defined by 5e-6 * "SignalSettings"."SampleFactor". (Remove last JOIN)
    # (2) SignalSettings is a small table, a query could be executed only once to get its content. Its information can be accessed locally after querying each one of the Blobs (time in RAM vs. SSH connection). (Remove First JOIN)

    cur.execute('SELECT DISTINCT("Blobs"."Id"), "Blobs"."DateTime", "Blobs"."Raw", "Settings"."SignalType", "Settings"."Unit", "Registers"."SystemSampleInterval"*"Settings"."SampleFactor" AS "SamplingInterval", "Blobs"."SignalSettingId" AS "dev" FROM "AQT"."SignalRecordsBlobs" AS "Blobs" INNER JOIN "AQT"."SignalSettings" AS "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" JOIN "AQT"."SignalRegisters" AS "Registers" ON "Blobs"."SignalRegisterId" = "Registers"."Id" WHERE "Blobs"."SignalSettingId" = %d AND "Blobs"."DateTime" BETWEEN \'%s\' AND \'%s\' AND "Blobs"."Id" = %d '%(SettingsId, t0, t1, _id))

    return cur.fetchall()

def convert_to_si(unit, val):
    switcher = {        # SI UNIT  , Conversion         | SI UNITS
        "rpm":  lambda: [0xE4923924, val/60.0],         # SI - freq - Hz --> 0xE4923924 1/sec
        "kVA":  lambda: [0xE49A9924, val*1000],         # 1000 * V * A = m^2*kg*s^⁻3*A^-1 * A = W
        "V":    lambda: [0xE49A9724, val],              # m^2*kg*s^⁻3*A^-1
        "m":    lambda: [0xE4964924, val],              # m
        "um":   lambda: [0xE4964924, val/1000000],      # m
        "%":    lambda: [0xF8000001, val],              # SI
        "bar":  lambda: [0xE48EA924, val*100000],       # Pa
        "MVAr": lambda: [0xE49A9924, val*1000000],      # 1000000 * V * A * r = W
        "g":    lambda: [0xE4962924, val*9.80665],      # g =  9.8 m/s^2
        "MW":   lambda: [0xE49A9924, val*1000000],      # m^2*kg*s^⁻3
        "A":    lambda: [0xE4924B24, val],              # A
        "oC":   lambda: [0xE4924964, val+273.15],       # K
        "mm":   lambda: [0xE4964924, val/1000],         # m
        "mm/s": lambda: [0xE4963924, val/1000],         # m/s
        "" :    lambda: [0x00000001, val]               # Digital switch
    }

    func = switcher.get(unit, lambda: ["Error", 0])
    return func()

def read_blob_float(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/4)
    for k in range(nv):
        lv.append(
            struct.unpack('f',bytearray(blob[k*4:(k*4+4)]))[0]
            )
    return lv

def read_blob_double(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/8)
    for k in range(nv):
        lv.append(
            struct.unpack('d',bytearray(blob[k*8:(k*8+8)]))[0]
            )
    return lv

def read_blob_bool(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob)))
    for k in range(nv):
        lv.append(
            struct.unpack('?',bytearray(blob[k:(k+1)]))[0]
            )
    return lv

def insert_data(rows, date_init, date_end, series_exists):
    # rows[i] =  [0] "Id", [1] "DateTime", [2] "Raw", [3] "SignalType", [4] "Unit", [5] "SamplingInterval", [6] "dev"
    # Steps:
    # 1. Convert data from blobl to bool/float/double list
    # 2. Convert data values in list to SI unit equivalent
    # 3. Write touple (value,ts) as a csv
    # 4. Write final string to a file and call PHP, when PHP returns, delete file
    # 4.1. PHP file name is "single-series-csv-import.php"
    # 4.2. PHP function is "insert_csv(filename, unit, x, y, z, dev, t0, tf, series_exist)"
    global file_num
    unit = convert_to_si(rows[0][4],0)[0]   # same unit for every data (each select is done for the same SignalSettingId)
    if (unit == "Error"):
         return "Not expected unit! rows[?][unit] = %s"%(rows[0][4])
    else:
        unit = int(unit)

    dev  = int(rows[0][6])                       # same dev for every data (each select is done for the same SignalSettingId == dev)

    t0 = int((date_init - datetime(1970, 1, 1)).total_seconds()*1000000)
    tf = int((date_end  - datetime(1970, 1, 1)).total_seconds()*1000000)

    str_csv = ""
    if (DEPLOYED):
        f = open("to_insert.csv","w")
    else:
        f = open("to_insert"+str(file_num)+".csv", "w")
        file_num += 1
    for i, r in enumerate(rows):
        timestamp = int((r[1] - datetime(1970, 1, 1)).total_seconds()*1000000) # micro
        if (r[3] == 1):
            list_values = read_blob_bool(r[2])
        elif(r[3] == 2):
            list_values = read_blob_float(r[2])
        elif(r[3] == 3):
            list_values = read_blob_double(r[2])
        else:
            return "Not expected type! rows[%d][type] = %d"%(i, r[3])
        for j, v in enumerate(list_values):
            unit_siv = convert_to_si(r[4], v)
            if (str(unit_siv[0]) == "Error"):
                return "Not expected unit! rows[%d][unit] = %s"%(i, r[4])
            str_csv = str(int(timestamp+(r[5]*j*1000000)))+','+str(unit_siv[1])+'\n'
            if ((i == len(rows)-1) and (j == len(list_values)-1)):
                str_csv = str_csv[:-1] #remove last "\n"
            f.write(str_csv)
        f.flush()
        f.close()
    if (DEPLOYED):
        #with open("to_insert.csv", "w") as f:
        #    f.write(str_csv[:-1]) #remove last "\n"
        if (series_exists):
            os.system("./single-series-csv-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(1)+" >> importer_php.log")
        else:
            os.system("./single-series-csv-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(0)+" >> importer_php.log")
        #code = "<?php include('./single-series-csv-import.php'); echo insert_csv(\"to_insert.csv\","+str(unit)+","+str(x)+","+str(y)+","+str(z)+","+str(dev)+","+str(t0)+","+str(tf)+","+'1' if series_exists else '0' + "); ?>"
        #code = "<?php include('./single-series-csv-import.php'); echo insert_csv(\"to_insert.csv\",0,0,0,0,0,0,0,1); ?>"
        #p = Popen(['php'], stdout=PIPE, stdin=PIPE, stderr=STDOUT, close_fds=True)

        # read output
        #res = str(p.communicate(code.encode('utf-8-sig'))[0])
        #print(res)
        # kill process
        #try:
        #    os.kill(p.pid, signal.SIGTERM)
        #except:
        #    pass

        #res = res.split("\n")[-1]
        #remove file
        #if (int(res) == 0):
        os.remove("to_insert.csv")
        #return "Error while inserting current file (error code = %d)"%(int(res)) if int(res) != 0 else "OK"
        return "Finished"
    else:
        print(str_csv[:-1])
        return "OK"


#settings_size = 122+1
#settings_size = 60
#settings_size = 56

setting_missing = [14, 15, 16, 17, 18]
date_init  = datetime(2019, 11, 24) # '2019-11-25' - 1 day
#date_init  = datetime(2020, 5, 31) # '2019-11-25' - 1 day
date_end   = datetime(2020, 9, 26) # '2020-09-25' + 1 day
delta = timedelta(hours=24)

#for setting in range(111, settings_size):
for setting in setting_missing:
    date_curr = date_init
    series_exists = False
    while date_curr <= date_end:
        t0 = date_curr.strftime('%Y-%m-%d')
        date_curr += delta
        t1 = date_curr.strftime('%Y-%m-%d')
        ids = query_postgress_for_rows_ids(setting, t0, t1)
        print("len IDS:",len(ids))
        if (len(ids) > 0):
            for _id in ids:
                rows = query_postgress(setting, t0, t1, int(_id[0]))
                print("len rows:",len(rows))
                if (len(rows) > 0):
                    out = insert_data(rows, date_init, date_end, series_exists)
                    if ("Not expected unit!" in out):
                        print(out)
                        break
                    print(out)
                    series_exists = True
