import psycopg2
from datetime import timedelta, datetime
import struct, sys, os
from subprocess import Popen, PIPE, STDOUT

DEPLOYED = True  # disable this flag to enable prints and disable insert at the DB
file_num = 1
lasts = {1:15, 2:73}  # Use this dictionary to set which Ids are already imported for each  SignalRecorderId (1 and 2). Only SignalIds greater than the value will be imported.

'''
"AQT"
-------------------------------
"SignalRecordsBlobs"
Id | SignalRegisterId | SignalSettingId | DateTime | Raw
-------------------------------
"SignalSettings"
Id | MonitoringElementId | SignalRecorderId | Name | Color | SampleFactor | SignalType | Unit
-------------------------------
"SignalRegisters"
Id | SignalRecorderId | StartDateTime | EndDateTime | IsSequentialRegister | SequentialNumberOfRegister | SystemSampleInterval
-------------------------------
"SignalRecorder"
Id |      Name      | SettingId 
----+----------------+-----------
  1 | Vibracao       |         1
  2 | Partida_Parada |         1
  1 | Vibracao       |         1
  2 | Partida_Parada |         1
(4 rows)
-------------------------------
test query (light version -- limiting time range and settingid)
select distinct("Blobs"."Id"), "Blobs"."DateTime", "Blobs"."Raw", "Settings"."SignalType", "Settings"."Unit", "Registers"."SystemSampleInterval"*"Settings"."SampleFactor" as "SamplingInterval" from "AQT"."SignalRecordsBlobs" as "Blobs" inner join "AQT"."SignalSettings" as "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" join "AQT"."SignalRegisters" as "Registers" ON "Blobs"."SignalRegisterId" = "Registers"."Id" where "Blobs"."SignalSettingId" = 2 and "Blobs"."DateTime" between '2019-10-28 00:00:00' and '2019-12-28 00:00:00';
'''

devs = {'EA1_Gerador-Traseiro_g' : 1, 'EA1_Gerador-Traseiro_g_RMS' : 2, 'EA1_Gerador-Traseiro_mm-s' : 3, 'EA1_Gerador-Traseiro_mm-s_RMS' : 4, 'EA2_Gerador-Dianteiro_g' : 5, 'EA2_Gerador-Dianteiro_g_RMS' : 6, 'EA2_Gerador-Dianteiro_mm-s' :  7, 'EA2_Gerador-Dianteiro_mm-s_RMS' : 8, 'EA3_Eixo-Alta-Velocidade_g' : 9, 'EA3_Eixo-Alta-Velocidade_g_RMS' : 10, 'EA3_Eixo-Alta-Velocidade_mm-s' : 11, 'EA3_Eixo-Alta-Velocidade_mm-s_RMS' : 12, 'EA4_2a-Planetaria_g' : 13, 'EA4_2a-Planetaria_g_RMS' : 14, 'EA4_2a-Planetaria_mm-s' : 15, 'EA4_2a-Planetaria_mm-s_RMS' : 16, 'EA5_1a-Planetaria_g' : 17, 'EA5_1a-Planetaria_g_RMS' : 18, 'EA5_1a-Planetaria_mm-s' : 19, 'EA5_1a-Planetaria_mm-s_RMS' : 20, 'EA6_Principal-100mV/g_g' : 21, 'EA6_Principal-100mV/g_g_RMS' : 22, 'EA6_Principal-100mV/g_mm-s' : 23, 'EA6_Principal-100mV/g_mm-s_RMS' : 24, 'EA7_Principal-500mV/g_g' : 25, 'EA7_Principal-500mV/g_g_RMS' : 26, 'EA7_Principal-500mV/g_mm-s' : 27, 'EA7_Principal-500mV/g_mm-s_RMS' : 28, 'EA8_Ref-fase' : 29, 'M10_Grid Current 1' : 30, 'M11_Grid Current 2' : 31, 'M12_Grid Current 3' : 32, 'M13_Environment Temp.' : 33, 'M14_Nacelle Temp.' : 34, 'M15_Pitch Generator Temp.1' : 35, 'M16_Pitch Generator Temp.2' : 36, 'M17_Pitch Generator Temp.3' : 37, 'M18_Gear Box Oil Temp.' : 38, 'M19_Gear Box Bearing Temp.' : 39, 'M1_Pitch angle 1' : 40, 'M20_Generator Bearing Tmep. Drive side' : 41, 'M21_Generator Bearing Tmep. Non-drive side' : 42, 'M22_Generator Coil Temp.1' : 43, 'M23_Generator Coil Temp.2' : 44, 'M24_Generator Coil Temp.3' : 45, 'M25_Wind Speed(1second)' : 46, 'M26_Win Direction' : 47, 'M27_Nacelle Position' : 48, 'M28_Nacelle Position demanded' : 49, 'M29_Active Power' : 50, 'M2_Pitch angle 2' : 51, 'M30_Reactive Power' : 52, 'M31_Rotor Speed' : 53, 'M32_Generator Speed' : 54, 'M33_Power400(400V consume)' : 55, 'M34_Water Warm temp.' : 56, 'M35_Water Cold temp.' : 57, 'M36_Converter GscIGBT temp.' : 58, 'M37_Converter LscIGBT temp.' : 59, 'M38_300 Cabinet temp.' : 60, 'M39_310 Cabinet temp.' : 61, 'M3_Pitch angle 3' : 62, 'M4_Pitch Torque 1' : 63, 'M5_Pitch Torque 2' : 64, 'M6_Pitch Torque 3' : 65, 'M7_Grid Voltage 1' : 66, 'M8_Grid Voltage 2' : 67, 'M9_Grid Voltage 3' : 68, 'RPM' : 69}

def query_postgres_for_rows_ids(SettingsId, t0, t1):
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech2', user='aqtech', password='KdeiBmtewpt')

    cur = conn.cursor()

    print("querying ids: ", SettingsId, t0, t1)

    # How to speed up by removing JOINs: (Slow speed up, the number of lines is usually small, blob size is the slowing factor)
    # (1) SystemSampleInterval seems to be a fixed value (at least it is for this dataset). The sampling interval can be defined by 5e-6 * "SignalSettings"."SampleFactor". (Remove last JOIN)
    # (2) SignalSettings is a small table, a query could be executed only once to get its content. Its information can be accessed locally after querying each one of the Blobs (time in RAM vs. SSH connection). (Remove First JOIN)

    cur.execute('SELECT DISTINCT("Blobs"."Id") FROM "AQT"."SignalRecordsBlobs" AS "Blobs" JOIN "AQT"."SignalSettings" AS "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" WHERE "Blobs"."SignalSettingId" = %d AND "Blobs"."DateTime" BETWEEN \'%s\' AND \'%s\' '%(SettingsId, t0, t1))

    return cur.fetchall()

def query_postgres(SettingsId, t0, t1, _blob_id):
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech2', user='aqtech', password='KdeiBmtewpt')

    cur = conn.cursor()

    print("query running", SettingsId, t0, t1, _blob_id)

    # How to speed up by removing JOINs: (Slow speed up, the number of lines is usually small, blob size is the slowing factor)
    # (1) SystemSampleInterval seems to be a fixed value (at least it is for this dataset). The sampling interval can be defined by 5e-6 * "SignalSettings"."SampleFactor". (Remove last JOIN)
    # (2) SignalSettings is a small table, a query could be executed only once to get its content. Its information can be accessed locally after querying each one of the Blobs (time in RAM vs. SSH connection). (Remove First JOIN)

    cur.execute('SELECT DISTINCT("Blobs"."Id"), "Blobs"."DateTime", "Blobs"."Raw", "Settings"."SignalType", "Settings"."Unit", "Registers"."SystemSampleInterval"*"Settings"."SampleFactor" AS "SamplingInterval", "Blobs"."SignalSettingId" AS "dev" FROM "AQT"."SignalRecordsBlobs" AS "Blobs" INNER JOIN "AQT"."SignalSettings" AS "Settings" ON "Blobs"."SignalSettingId" = "Settings"."Id" JOIN "AQT"."SignalRegisters" AS "Registers" ON "Blobs"."SignalRegisterId" = "Registers"."Id" WHERE "Blobs"."SignalSettingId" = %d AND "Blobs"."DateTime" BETWEEN \'%s\' AND \'%s\' AND "Blobs"."Id" = %d '%(SettingsId, t0, t1, _blob_id))

    return cur.fetchall()

def convert_to_si(unit, val):
    switcher = {        # SI UNIT  , Conversion         | SI UNITS
        "RPM":  lambda: [0xE4923924, val/60.0],         # SI - freq - Hz --> 0xE4923924 1/sec
        "rpm":  lambda: [0xE4923924, val/60.0],         # SI - freq - Hz --> 0xE4923924 1/sec
        "kVA":  lambda: [0xE49A9924, val*1000],         # 1000 * V * A = m^2*kg*s^⁻3*A^-1 * A = W
        "V":    lambda: [0xE49A9724, val],              # m^2*kg*s^⁻3*A^-1
        "m":    lambda: [0xE4964924, val],              # m
        "um":   lambda: [0xE4964924, val/1000000],      # m
        "%":    lambda: [0xF8000001, val],              # SI
        "bar":  lambda: [0xE48EA924, val*100000],       # Pa
        "MVAr": lambda: [0xE49A9924, val*1000000],      # 1000000 * V * A * r = W
        "kVAr": lambda: [0xE49A9924, val*1000],         # 1000 * V * A * r = W
        "g":    lambda: [0xE4962924, val*9.80665],      # g =  9.8 m/s^2
        "MW":   lambda: [0xE49A9924, val*1000000],      # m^2*kg*s^⁻3
        "kW":   lambda: [0xE49A9924, val*1000],         # m^2*kg*s^⁻3
        "A":    lambda: [0xE4924B24, val],              # A
        "oC":   lambda: [0xE4924964, val+273.15],       # K
        "mm":   lambda: [0xE4964924, val/1000],         # m
        "mm/s": lambda: [0xE4963924, val/1000],         # m/s
        "m/s":  lambda: [0xE4963924, val],	        # m/s, no conversion
        "Nm":   lambda: [0xE49AA924, val],              # 1N = ( Kg * m / s2 ) * m = Kg * m2 / s2 
        "o" :   lambda: [0xE4B24924, val*0.017453293],  # 1 degree -> 0.017453293 radians
        "" :    lambda: [0x00000001, val]               # Digital switch
    }
    func = switcher.get(unit, lambda: ["Error", 0])
    return func()

def read_blob_float(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/4)
    for k in range(nv):
        lv.append(
            struct.unpack('f',bytearray(blob[k*4:(k*4+4)]))[0]
            )
    return lv

def read_blob_double(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob))/8)
    for k in range(nv):
        lv.append(
            struct.unpack('d',bytearray(blob[k*8:(k*8+8)]))[0]
            )
    return lv

def read_blob_bool(blob):
    # List variable that serves as a dumpster for the values
    lv = []
    # Number of values on the record
    nv = int(len(bytearray(blob)))
    for k in range(nv):
        lv.append(
            struct.unpack('?',bytearray(blob[k:(k+1)]))[0]
            )
    return lv

def insert_data(rows, date_init, date_end, series_exists, _dev):
    # rows[i] =  [0] "Id", [1] "DateTime", [2] "Raw", [3] "SignalType", [4] "Unit", [5] "SamplingInterval", [6] "dev"
    # Steps:
    # 1. Convert data from blobl to bool/float/double list
    # 2. Convert data values in list to SI unit equivalent
    # 3. Write touple (value,ts) as a csv
    # 4. Write final string to a file and call PHP, when PHP returns, delete file
    # 4.1. PHP file name is "single-series-csv-import.php"
    # 4.2. PHP function is "insert_csv(filename, unit, x, y, z, dev, t0, tf, series_exist)"
    global file_num
    unit = convert_to_si(rows[0][4],0)[0]   # same unit for every data (each select is done for the same SignalSettingId)
    if (unit == "Error"):
         return "Not expected unit! rows[?][unit] = %s"%(rows[0][4])
    else:
        unit = int(unit)

    dev = _dev      # same dev for every data (each select is done for the same SignalSettingId == dev)

    t0 = int((date_init - datetime(1970, 1, 1)).total_seconds()*1000000)
    tf = int((date_end  - datetime(1970, 1, 1)).total_seconds()*1000000)

    str_csv = ""
    if (DEPLOYED):
        f = open("to_insert.csv","w")
    else:
        f = open("to_insert"+str(file_num)+".csv", "w")
        file_num += 1
    for i, r in enumerate(rows):
        timestamp = int((r[1] - datetime(1970, 1, 1)).total_seconds()*1000000) # micro
        if (r[3] == 1):
            list_values = read_blob_bool(r[2])
        elif(r[3] == 2):
            list_values = read_blob_float(r[2])
        elif(r[3] == 3):
            list_values = read_blob_double(r[2])
        else:
            return "Not expected type! rows[%d][type] = %d"%(i, r[3])
        for j, v in enumerate(list_values):
            unit_siv = convert_to_si(r[4], v)
            if (str(unit_siv[0]) == "Error"):
                return "Not expected unit! rows[%d][unit] = %s"%(i, r[4])
            str_csv = str(int(timestamp+(r[5]*j*1000000)))+','+str(unit_siv[1])+'\n'
            if ((i == len(rows)-1) and (j == len(list_values)-1)):
                str_csv = str_csv[:-1] #remove last "\n"
            f.write(str_csv)
        f.flush()
        f.close()
    if (DEPLOYED):
        #with open("to_insert.csv", "w") as f:
        #    f.write(str_csv[:-1]) #remove last "\n"
        if (series_exists):
            os.system("./single-series-csv-aqtech-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(1)+" >> importer2_php.log")
        else:
            os.system("./single-series-csv-aqtech-import.php 'to_insert.csv' "+str(unit)+" "+str(x)+" "+str(y)+" "+str(z)+" "+str(dev)+" "+str(t0)+" "+str(tf)+" "+str(0)+" >> importer2_php.log")

        os.remove("to_insert.csv")
        #return "Error while inserting current file (error code = %d)"%(int(res)) if int(res) != 0 else "OK"
        return "Finished"
    else:
        print(str_csv[:-1])
        return "OK"

delta = timedelta(hours=24)
for tb in [2]:   # Excluded 2, as AQTech reported that it was an error, should be only one SignalRecorder
    if tb == 1:
        x = 317500000
        y = -426100000
        z = -351400000
    else:
        x = 317599000
        y = -426199000
        z = -351499000
    last_import = lasts[tb]
    conn = psycopg2.connect(host='150.162.62.97', port=5432, database='aqtech2', user='aqtech', password='KdeiBmtewpt')
    cur = conn.cursor()
    cur.execute('select "Id", "Name", "Unit" from "AQT"."SignalSettings" where "SignalRecorderId" = %d and "Id" > %d order by "Id"'%(tb,last_import))
    rows = cur.fetchall()
    for r in rows:
        date_init  = datetime(2019, 5, 1) # '2019-11-25' - 1 day
        date_end   = datetime(2019, 10, 30) # '2020-09-25' + 1 day
        date_curr = date_init
        series_exists = False
        while date_curr <= date_end:
            t0 = date_curr.strftime('%Y-%m-%d')
            date_curr += delta
            t1 = date_curr.strftime('%Y-%m-%d')
            _id = r[0]
            _name = r[1]
            _unit = r[2]
            _ndev = devs[_name]
            _dev = 6000 + _ndev
            print(_id, _name, _unit, _dev)
            ids = query_postgres_for_rows_ids(_id, t0, t1)
            print("  ==> Importing SignalSettinsId = ", _id, " from ", t0, ' to ', t1); 
            print("len IDS:",len(ids))
            if (len(ids) > 0):
                for _bid in ids:
                    rows = query_postgres(_id, t0, t1, int(_bid[0]))
                    print("len rows:",len(rows))
                    if (len(rows) > 0):
                        out = insert_data(rows, date_init, date_end, series_exists, _dev)
                        if ("Not expected unit!" in out):
                            print(out)
                            break
                        print(out)
                        series_exists = True
        print("  ==> Importing ", _id," finished")
