#!/bin/bash
# Caciano Machado/Juliano Zatta - 16/10/2017

cd /certmanager

. config

cd certificates

# Initilize if not available the certificate list
NUMPREVCERT=`tail -n 1 certificate_list | cut -d " " -f 1`
ISNUM='^[0-9]+$'

# Update - 30-10-2020 - Roberto M. Scheffel - Added receiver and purpose description, for documentation

receiver=$1
descr=$2
if [ -z "$receiver" ]
then
   echo 'Usage: ' $0 '<receiver name>' '<purpose short description>.'
   exit
fi
if [ -z "$descr" ]
then
   echo 'Usage: ' $0 '<receiver name>' '<purpose short description>.'
   exit
fi

if [[ -a certificate_list ]]; then
    if ! [[ $NUMPREVCERT =~ $ISNUM ]]; then
        echo "ERROR: File certificate_list corrupted. This file stores the list of generated certificates and respective serial numbers."
        exit -1
    fi
    CURRCLIENT=$((NUMPREVCERT + 1))
    LASTCLIENT=$((NUMCLIENTS + NUMPREVCERT))
else
    CURRCLIENT=1
    LASTCLIENT=$NUMCLIENTS
fi

if [ ! -f /certmanager/certificates/rootCA.srl ]; then
    echo `openssl rand -hex 8` > /certmanager/certificates/rootCA.srl
    ln -s /certmanager/certificates/rootCA.srl /certmanager/rootCA.srl
fi


for i in `seq $CURRCLIENT $LASTCLIENT`; do    
    # Generate RSA key

    echo "openssl genrsa -out client-$i.key $KEYSIZE"
    openssl genrsa -out client-$i.key $KEYSIZE

    # Generate certificate request
    echo "openssl req -subj \"/C=BR/ST=$STATE/L=$CITY/O=$ORG/OU=$UNIT/CN=$HOST\" -new -key client-$i.key -out client-$i.req"
    openssl req -subj "/C=BR/ST=$STATE/L=$CITY/O=$ORG/OU=$UNIT/CN=$HOST" -new -key client-$i.key -out client-$i.req
    # Generate certificate
    echo "openssl x509 -req -in client-$i.req -CA ${CERTNAME}.crt -CAkey ${CERTNAME}.key -CAcreateserial -out client-$i.pem -days $CLIENTEXP -sha256"
    openssl x509 -req -in client-$i.req -CA ${CERTNAME}.crt -CAkey ${CERTNAME}.key -CAcreateserial -out client-$i.pem -days $CLIENTEXP -sha256
    serial=`openssl x509 -in client-$i.pem -serial -noout | cut -d "=" -f 2`
    echo $i $serial
    echo $i $serial `date "+%F-%T"` `whoami` ' | ' $receiver ' | ' $descr >> certificate_list
    mv client-$i.key client-$i-$serial.key
    mv client-$i.pem client-$i-$serial.pem
    rm -f client-$i.req
    echo "$serial client-$i-$serial.pem client-$i-$serial.key"
done