#!/bin/bash

# Get the directory of the script
SCRIPT_DIR="$(dirname "$(realpath "$BASH_SOURCE[0]")")"

# Go to the parent directory
PARENT_DIR="$(dirname "$SCRIPT_DIR")"

# Change to the parent directory
cd "$PARENT_DIR"

# Get the list of services and their status
services=$(docker compose --profile test ps --services)

# Initialize a flag for checking health
all_healthy=true

# Loop through each service
for service in $services; do
    # Get the container ID of the service
    container_id=$(docker compose --profile test ps -q "$service")

    # Check if the service has a health status
    health_status=$(docker inspect --format='{{.State.Health.Status}}' "$container_id" 2>/dev/null)

    if [ "$health_status" == "" ]; then
        # No health check defined, check if the container is running
        status=$(docker inspect --format='{{.State.Status}}' "$container_id")
        if [ "$status" != "running" ]; then
            echo "$service is not running. Status: $status"
            all_healthy=false
        else
            echo "$service is running."
        fi
    else
        # Health check is defined, use its status
        if [ "$health_status" != "healthy" ]; then
            echo "$service is not healthy. Health status: $health_status"
            all_healthy=false
        else
            echo "$service is healthy."
        fi
    fi
done

# Final message and exit status
if [ "$all_healthy" = true ]; then
    echo "All services are up and healthy."
    exit 0
else
    echo "Some services are not healthy or not running."
    exit 1
fi