#!/bin/bash

# Configuration
CA_KEY="ca.key"
CA_CERT="ca.crt"
SERVER_KEY="server.key"
SERVER_CSR="server.csr"
SERVER_CERT="server.crt"
DAYS_VALIDITY=3650

# 1. Create CA Private Key without passphrase
openssl genpkey -algorithm RSA -out "$CA_KEY"

# 2. Create CA Certificate
openssl req -new -x509 -key "$CA_KEY" -out "$CA_CERT" -days "$DAYS_VALIDITY"

# 3. Create Server Private Key without passphrase
openssl genpkey -algorithm RSA -out "$SERVER_KEY"

# 4. Create Server Certificate Signing Request (CSR)
openssl req -new -key "$SERVER_KEY" -out "$SERVER_CSR"

# 5. Sign Server Certificate with CA
openssl x509 -req -in "$SERVER_CSR" -CA "$CA_CERT" -CAkey "$CA_KEY" -CAcreateserial -out "$SERVER_CERT" -days "$DAYS_VALIDITY"

# Cleanup temporary files (optional)
rm "$SERVER_CSR" "$CA_KEY.srl"

echo "Certificate Authority (CA) certificate created: $CA_CERT"
echo "Server Private Key created: $SERVER_KEY"
echo "Server Certificate signed by CA created: $SERVER_CERT"

