from keras.layers import Input, Dropout
from keras.layers.core import Dense 
from keras.models import Model, Sequential, load_model
from keras import regularizers
from keras.models import model_from_json
from keras.callbacks import EarlyStopping
import numpy as np

class Deep_Autoencoder:
    def __init__(self, input_layer=0, output_layer=0, encode_layers=[0], decode_layers=[0], encode_activations=['relu'], 
                    encode_kernel_regularizers=[regularizers.l2(0.1)], encode_kernel_initializer=['glorot_uniform'],
                    decode_activations=['relu'], 
                    decode_kernel_regularizers=[regularizers.l2(0.1)], decode_kernel_initializer=['glorot_uniform'],
                    load_file="",optimizer='adam', loss = 'mse'):
        if (load_file != ""):
            self.auto_encoder = load_model(load_file+".h5")
        else:
            self.auto_encoder = Sequential()

            self.auto_encoder.add(Dense(units=encode_layers[0], activation=encode_activations[0], 
                                    input_dim=input_layer, kernel_regularizer=encode_kernel_regularizers[0], kernel_initializer=encode_kernel_initializer[0]))
            act = 0
            regu = 0
            init = 0
            for layer in range(1,len(encode_layers)):
                if (len(encode_activations) > 1):
                    act +=1
                #if (len(encode_kernel_regularizers) > 1):
                #    regu += 1
                if (len(encode_kernel_initializer) > 1):
                    init += 1
                self.auto_encoder.add(Dense(units=encode_layers[layer], activation=encode_activations[act],
                                kernel_initializer=encode_kernel_initializer[init]))

            act = 0
            regu = 0
            init = 0
            for layer in range(0,len(decode_layers)):
                self.auto_encoder.add(Dense(units=decode_layers[layer], activation=decode_activations[act],
                                kernel_initializer=decode_kernel_initializer[init]))
                if (len(decode_activations) > 1):
                    act +=1
                #if (len(decode_kernel_regularizers) > 1):
                #    regu += 1
                if (len(decode_kernel_initializer) > 1):
                    init += 1

            self.auto_encoder.compile(optimizer=optimizer, loss=loss)

    def train(self, X_train, epochs, batch_size, X_test, callbacks):
        return self.auto_encoder.fit(np.array(X_train),np.array(X_train),
                epochs = epochs, batch_size = batch_size, 
                validation_data = (X_test, X_test), callbacks=callbacks)

    def predict(self, X):
        #print(X)
        return self.auto_encoder.predict(X)

    def save(self, file_name):
        self.auto_encoder.save(file_name+".h5")

