from common.PlatformManagement import PlatformManagement
from common.SmartdataClient import SmartdataClient
from hashlib import sha256
import time
# Initializing Test Environment
client = PlatformManagement()
domain = 'anontest' + str(int(time.time()))
assert client.createDomain(domain), 'Domain was not created'
cert = client.generateCredentials(domain)
assert cert['serial'], 'Cert was not created'
with open("/tmp/client.key", "w", encoding="utf-8") as file:
    file.write(str(cert['key']))
with open("/tmp/client.crt", "w", encoding="utf-8") as file:
    file.write(str(cert['pem']))

client = SmartdataClient()

# Criar duas series com sigs diferentes
client.createSeries(unit=0x84963924, x=0, y=0, z=0, r=100, t0=0, t1=10000, dev=1, signature="sig1")
client.createSeries(unit=0x84963924, x=0, y=0, z=0, r=100, t0=0, t1=10000, dev=1, signature="sig2")

# Adicionar o dado na sig1
client.put(unit=0x84963924, value=100, x=10, y=10, z=10, t=10, dev=1, signature="sig1")
client.put(unit=0x84963924, value=100, x=11, y=11, z=11, t=11, dev=1, signature="sig1")
client.put(unit=0x84963924, value=100, x=12, y=12, z=12, t=12, dev=1, signature="sig1")

# Adicionar o dado na sig2
client.put(unit=0x84963924, value=100, x=20, y=20, z=20, t=10, dev=1, signature="sig2")
client.put(unit=0x84963924, value=100, x=21, y=21, z=21, t=11, dev=1, signature="sig2")
client.put(unit=0x84963924, value=100, x=22, y=22, z=22, t=12, dev=1, signature="sig2")

# Busca pela signature "sig1" (que é armazenada como HASH)
r = client.get(unit=0x84963924, x=10, y=10, z=10, r=100, t0=10, tf=20, dev=1, signature="sig1")
# x,y,z bate com original?
assert r['series'], f"Deveria ter retornado registros, mas retornou {r}"
assert len(r['series']) == 3, f"Deveria ter retornado 3 registros, mas retornou {r}"
assert r['series'][0]['x'] == 10, f"Valor de xyz retornou errado: {r}"
assert r['series'][1]['y'] == 11, f"Valor de xyz retornou errado: {r}"
assert r['series'][2]['z'] == 12, f"Valor de xyz retornou errado: {r}"
# Signature bate com a enviada na query?
assert r['series'][0]["signature"] == "sig1", f"Deveria ter retornado registros com signature sig1: {r}"

# GET pelo HASH da signature para testar a retrocompatibilidade
hash = sha256("sig1".encode()).hexdigest()
r = client.get(unit=0x84963924, x=10, y=10, z=10, r=100, t0=10, tf=20, dev=1, signature=hash)
# x,y,z bate com original?
assert r['series'], f"Deveria ter retornado registros, mas retornou {r}"
assert len(r['series']) == 3, f"Deveria ter retornado 3 registros, mas retornou {r}"
assert r['series'][0]['x'] == 10, f"Valor de xyz retornou errado: {r}"
assert r['series'][1]['y'] == 11, f"Valor de xyz retornou errado: {r}"
assert r['series'][2]['z'] == 12, f"Valor de xyz retornou errado: {r}"
# Signature bate com original?
assert r['series'][0]["signature"] == hash, "Deveria ter retornado registros com hash da signature"

# GET sem signature
r = client.get(unit=0x84963924, x=10, y=10, z=10, r=100, t0=10, tf=20, dev=1)
# voltou duas series?
assert r['series'], f"Deveria ter retornado registros, mas retornou {r}"
assert len(r['series']) == 6, f"Deveria ter retornado 6 registros, mas retornou {r}"

# signatures foram anonimizada?
hash1 = sha256("sig1".encode()).hexdigest()
hash2 = sha256("sig2".encode()).hexdigest()
# Retornou duas signatures diferentes?
sigs = [0, 0]
for s in r['series']:
    sigs[s['signature']] = sigs[s['signature']] + 1
assert sigs[0] == 3, f'Deveria ter retornado 3 signatures "0": {sigs}'
assert sigs[1] == 3, f'Deveria ter retornado 3 signatures "1": {sigs}'

# x,y,z foram anonimizadas
# Obs: no teste, a cada tempo t o x,y,z incrementam em 1 na sig1 e 10 na sig2 para facilitar o teste
d = r['series']
idx = [0, 1, 2, 3, 4, 5]
# Valores de xyz anonimizados que esperamos
xyz = [0, 1, 2, 10, 11, 12]
for i in idx:
    for j in ['x', 'y', 'z']:
        assert d[i][j] == xyz[i], f"[S{i}] Deveria ter retornado o {j} of {xyz[i]}: {d[i]}"
