import requests
import json
import time


class PlatformManagement:
    def __init__(self, url="http://management-api", api_key="testtoken"):
        self._url = url
        self._api_key = api_key
        self._client = requests.Session()
        self._client.cookies.set('API_KEY', api_key)

    def _doGet(self, endpoint="", params={}):
        url = self._url + endpoint
        response = self._client.get(url, params=params)
        # print ("R: " + response.text )
        return response if response.status_code == 200 else False

    def listDomains(self):
        r = self._doGet('/v1/domains')
        return json.loads(r.text) if r else False

    def clearDomain(self, domain):
        r = self._doGet(f'/v1/domains/clear/{domain}')
        return True if r else False

    def createDomain(self, domain):
        r = self._doGet(f'/v1/domains/create/{domain}')
        return True if r else False

    def createCertificate(self):
        r = self._doGet('/v1/domains/credentials')
        return json.loads(r.text) if r else False

    def associateCertificate(self, domain, serial):
        r = self._doGet(f'/v1/domains/addcertificate/{domain}/{serial}')
        return True if r else False

    def deleteDomain(self, domain):
        r = self._doGet(f'/v1/domains/delete/{domain}')
        return json.loads(r.text) if r else False

    def domainExists(self, domain):
        r = self._doGet(f'/v1/domains/exists/{domain}')
        return json.loads(r.text) if r else False

    def generateCredentials(self, domain):
        cert = self.createCertificate()
        if not cert:
            return False
        assoc = self.associateCertificate(domain, cert['serial'])
        return cert if assoc else False
