import requests
import json


class SmartdataClient:
    def __init__(
            self,
            cert_file='/tmp/client.crt',
            key_file='/tmp/client.key',
            url="https://web/",
            logfile=None,
            verifyCertificate=False):
        if not url.endswith('/'):
            url += '/'
        self._url = url
        self._client = requests.Session()
        self._client.cert = (cert_file, key_file)
        self._client.verify = verifyCertificate
        self._logfile = logfile
        if not verifyCertificate:
            requests.urllib3.disable_warnings()

    def _doJsonPost(self, endpoint, json_data, return_attribute=''):
        headers = {'Content-Type': 'application/json'}
        url = self._url + endpoint
        response = self._client.post(url, headers=headers, json=json_data)
        return response if response.status_code < 300 else False

    def createSeries(self, unit, x, y, z, r, t0, t1, dev, signature, workflow=None, version="1.2"):
        json_request = {
            "series": {
                "version": version,
                "unit": unit,
                "x": x,
                "y": y,
                "z": z,
                "r": r,
                "t0": t0,
                "t1": t1,
                "dev": dev,
                "signature": signature
            }
        }

        if workflow is not None:
            json_request["request"]["workflow"] = workflow

        r = self._doJsonPost("/api/create.php", json_request)

        return True if r else False

    def get(self, unit, x, y, z, r, t0, tf, dev, signature=None, version="1.2"):
        json_request = {
            "series": {
                "version": version,
                "unit": unit,
                "dev": dev,
                "x": x,
                "y": y,
                "z": z,
                "r": r,
                "t0": t0,
                "tf": tf
            }
        }
        if signature is not None:
            json_request["series"]["signature"] = signature

        result = self._doJsonPost("/api/get.php", json_request)
        return result.json() if len(result.text) > 1 else False

    def put(self, unit, value, x, y, z, t, dev, signature, error=0, confidence=1, workflow=0, version="1.2"):
        json_request = {
            "smartdata": [
                {
                    "version": version,
                    "unit": unit,
                    "value": value,
                    "error": error,
                    "confidence": confidence,
                    "signature": signature,
                    "x": x,
                    "y": y,
                    "z": z,
                    "r": 0,
                    "t": t,
                    "dev": dev,
                    "workflow": 0
                }
            ]
        }
        return self._doJsonPost("/api/put.php", json_request)
